package by.zerg.psim;

import java.io.File;

import javax.swing.SwingUtilities;

import by.zerg.psim.gui.ImagePanelListener;
import by.zerg.psim.gui.MainFrame;
import by.zerg.psim.gui.MainFrameListener;
import by.zerg.psim.gui.Messenger;
import by.zerg.psim.gui.PokerStarsImageChooser;
import by.zerg.psim.gui.PokerStarsImageChooserListener;
import by.zerg.psim.helpers.BinExecutor;
import by.zerg.psim.model.Config;
import by.zerg.psim.model.ImageManager;

public class MainClass implements MainFrameListener,
		PokerStarsImageChooserListener, ImagePanelListener {
	MainFrame mainFrame;
	PokerStarsImageChooser imageChooser;
	ImageManager imageManager;

	private MainClass() {
		mainFrame = new MainFrame();
		mainFrame.getMainImagePanel().addListener(this);
		mainFrame.getMaskImagePanel().addListener(this);
		imageChooser = new PokerStarsImageChooser();
		imageChooser.addListener(this);
		imageManager = new ImageManager();
		mainFrame.addListener(this);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Throwable {
		// try {
		// javax.swing.UIManager.LookAndFeelInfo[] installedLookAndFeels =
		// javax.swing.UIManager
		// .getInstalledLookAndFeels();
		// for (int idx = 0; idx < installedLookAndFeels.length; idx++) {
		// if ("Nimbus".equals(installedLookAndFeels[idx].getName())) {
		// javax.swing.UIManager
		// .setLookAndFeel(installedLookAndFeels[idx]
		// .getClassName());
		// break;
		// }
		// }
		// } catch (Throwable t) {
		// }

		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				new MainClass().mainFrame.init();
			}

		});
	}

	@Override
	public void mainImageSelected(File mainImageFile) {
		try {
			imageManager.setMainImage(mainImageFile);
			mainFrame.getMainImagePanel().setImage(
					imageManager.getCurrentMainImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			return;
		}
		if (Config.getInstance().isAutoOpenAssociatedImages()) {
			try {
				File foundMaskImageFile = imageManager
						.findMaskImageFile(mainImageFile);
				imageManager.setMaskImage(foundMaskImageFile);
				mainFrame.getMaskImagePanel().setImage(
						imageManager.getCurrentMaskImage());
			} catch (Throwable t) {
				Messenger.getInstance().showMessageDialog(mainFrame,
						t.getMessage());
				imageManager.clearMaskImage();
				mainFrame.getMaskImagePanel().setImage(null);
			}
		}
		try {
			mainFrame.getResultImagePanel().setImage(
					imageManager.getCurrentResultImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			mainFrame.getResultImagePanel().setImage(null);
		}
	}

	@Override
	public void maskImageSelected(File maskImageFile) {
		try {
			imageManager.setMaskImage(maskImageFile);
			mainFrame.getMaskImagePanel().setImage(
					imageManager.getCurrentMaskImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			return;
		}
		if (Config.getInstance().isAutoOpenAssociatedImages()) {
			try {
				File foundMainImageFile = imageManager
						.findMainImageFile(maskImageFile);
				imageManager.setMainImage(foundMainImageFile);
				mainFrame.getMainImagePanel().setImage(
						imageManager.getCurrentMainImage());
			} catch (Throwable t) {
				Messenger.getInstance().showMessageDialog(mainFrame,
						t.getMessage());
				imageManager.clearMainImage();
				mainFrame.getMainImagePanel().setImage(null);
			}
		}
		try {
			mainFrame.getResultImagePanel().setImage(
					imageManager.getCurrentResultImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			mainFrame.getResultImagePanel().setImage(null);
		}
	}

	@Override
	public void imageSizingChanged() {
		// TODO Auto-generated method stub

	}

	@Override
	public void refreshButtonClicked() {
		// TODO Auto-generated method stub

	}

	@Override
	public void convert2bmpButtonClicked() {
		// TODO Auto-generated method stub

	}

	@Override
	public void convert2pngButtonClicked() {
		imageManager.convert2png(new File(imageManager
				.getSelectedMainImageFile().getAbsolutePath().toLowerCase()
				.replace(".bmp", ".png")));
	}

	@Override
	public void extractMaskButtonClicked() {
		// TODO Auto-generated method stub

	}

	@Override
	public void openMainImageButtonClicked() {
		imageChooser.showOpenMainImageDialog(mainFrame);
	}

	@Override
	public void findMainImageButtonClicked() {
		BinExecutor.execute(Config.getInstance().getNavigatorCommand() + " \""
				+ imageManager.getSelectedMainImageFile().getAbsolutePath()
				+ "\"");
	}

	@Override
	public void editMainImageButtonClicked() {
		mainFrame.getRootPane().getGlassPane().setVisible(true);
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				BinExecutor.execute(Config.getInstance().getEditorCommand()
						+ " \""
						+ imageManager.getEditableMainImageFile()
								.getAbsolutePath() + "\"");
				mainFrame.getRootPane().getGlassPane().setVisible(false);
			}

		});
	}

	@Override
	public void refreshMainImageButtonClicked() {
		try {
			imageManager.refreshMainImage();
			mainFrame.getMainImagePanel().setImage(
					imageManager.getCurrentMainImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			imageManager.clearMainImage();
			mainFrame.getMainImagePanel().setImage(null);
		}
		try {
			mainFrame.getResultImagePanel().setImage(
					imageManager.getCurrentResultImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			mainFrame.getResultImagePanel().setImage(null);
		}
	}

	@Override
	public void openMaskImageButtonClicked() {
		imageChooser.showOpenMaskImageDialog(mainFrame);
	}

	@Override
	public void findMaskImageButtonClicked() {
		BinExecutor.execute(Config.getInstance().getNavigatorCommand() + " \""
				+ imageManager.getSelectedMaskImageFile().getAbsolutePath()
				+ "\"");
	}

	@Override
	public void editMaskImageButtonClicked() {
		mainFrame.getRootPane().getGlassPane().setVisible(true);
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				BinExecutor.execute(Config.getInstance().getEditorCommand()
						+ " \""
						+ imageManager.getEditableMaskImageFile()
								.getAbsolutePath() + "\"");
				mainFrame.getRootPane().getGlassPane().setVisible(false);
			}

		});
	}

	@Override
	public void refreshMaskImageButtonClicked() {
		try {
			imageManager.refreshMaskImage();
			mainFrame.getMaskImagePanel().setImage(
					imageManager.getCurrentMaskImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			imageManager.clearMaskImage();
			mainFrame.getMaskImagePanel().setImage(null);
		}
		try {
			mainFrame.getResultImagePanel().setImage(
					imageManager.getCurrentResultImage());
		} catch (Throwable t) {
			Messenger.getInstance()
					.showMessageDialog(mainFrame, t.getMessage());
			mainFrame.getResultImagePanel().setImage(null);
		}
	}

	@Override
	public void openButtonClicked() {
		// TODO Auto-generated method stub
		
	}
}
