package by.zerg.psim.gui;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;

import by.zerg.psim.helpers.ImageHelper;

public class PokerStarsImageChooser {
	JFileChooser openFileChooser;
	JFileChooser mainImageFileChooser;
	JFileChooser maskImageFileChooser;
	FileFilter png8BitsFilter;
	FileFilter png24BitsFilter;
	FileFilter png32BitsFilter;
	FileFilter bmp24BitsFilter;
	FileFilter bmp8BitsFilter;
	FileFilter allImagesFilter;

	List<PokerStarsImageChooserListener> listeners;

	public PokerStarsImageChooser() {
		listeners = new ArrayList<PokerStarsImageChooserListener>();

		png8BitsFilter = new FileFilter() {

			@Override
			public String getDescription() {
				return "PNG (8 bits)";
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				try {
					return new ImageHelper(f).getImageType() == ImageHelper.PNG_8;
				} catch (Throwable t) {
					return false;
				}
			}
		};

		png24BitsFilter = new FileFilter() {

			@Override
			public String getDescription() {
				return "PNG (24 bits)";
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				try {
					return new ImageHelper(f).getImageType() == ImageHelper.PNG_24;
				} catch (Throwable t) {
					return false;
				}
			}
		};

		png32BitsFilter = new FileFilter() {

			@Override
			public String getDescription() {
				return "PNG (32 bits)";
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				try {
					return new ImageHelper(f).getImageType() == ImageHelper.PNG_32;
				} catch (Throwable t) {
					return false;
				}
			}
		};

		bmp8BitsFilter = new FileFilter() {

			@Override
			public String getDescription() {
				return "BMP (8 bits)";
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				try {
					return new ImageHelper(f).getImageType() == ImageHelper.BMP_8;
				} catch (Throwable t) {
					return false;
				}
			}
		};

		bmp24BitsFilter = new FileFilter() {

			@Override
			public String getDescription() {
				return "BMP (24 bits)";
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				try {
					return new ImageHelper(f).getImageType() == ImageHelper.BMP_24;
				} catch (Throwable t) {
					return false;
				}
			}
		};

		allImagesFilter = new FileFilter() {

			@Override
			public String getDescription() {
				return "All Possible Images";
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				try {
					switch (new ImageHelper(f).getImageType()) {
					case ImageHelper.BMP_8:
					case ImageHelper.BMP_24:
					case ImageHelper.PNG_24:
					case ImageHelper.PNG_32:
						return true;
					default:
						return false;
					}
				} catch (Throwable t) {
					return false;
				}
			}
		};

		openFileChooser = new JFileChooser();
		openFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		openFileChooser.setMultiSelectionEnabled(false);
		openFileChooser.setAcceptAllFileFilterUsed(false);
		openFileChooser.addChoosableFileFilter(png32BitsFilter);
		openFileChooser.addChoosableFileFilter(png24BitsFilter);
		openFileChooser.addChoosableFileFilter(bmp24BitsFilter);
		openFileChooser.addChoosableFileFilter(allImagesFilter);

		mainImageFileChooser = new JFileChooser();
		mainImageFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		mainImageFileChooser.setMultiSelectionEnabled(false);
		mainImageFileChooser.setAcceptAllFileFilterUsed(false);
		mainImageFileChooser.addChoosableFileFilter(png32BitsFilter);
		mainImageFileChooser.addChoosableFileFilter(png24BitsFilter);
		mainImageFileChooser.addChoosableFileFilter(png8BitsFilter);
		mainImageFileChooser.addChoosableFileFilter(bmp24BitsFilter);
		mainImageFileChooser.addChoosableFileFilter(bmp8BitsFilter);
		mainImageFileChooser.addChoosableFileFilter(allImagesFilter);

		maskImageFileChooser = new JFileChooser();
		maskImageFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		maskImageFileChooser.setMultiSelectionEnabled(false);
		maskImageFileChooser.setAcceptAllFileFilterUsed(false);
		maskImageFileChooser.addChoosableFileFilter(bmp8BitsFilter);
	}

	public void addListener(PokerStarsImageChooserListener listener) {
		listeners.add(listener);
	}

	public void showOpenDialog(JFrame parent) {
		parent.getRootPane().getGlassPane().setVisible(true);
		if (mainImageFileChooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) {
			for (PokerStarsImageChooserListener listener : listeners) {
				listener.mainImageSelected(mainImageFileChooser
						.getSelectedFile());
			}
		}
		parent.getRootPane().getGlassPane().setVisible(false);
	}

	public void showOpenMainImageDialog(JFrame parent) {
		parent.getRootPane().getGlassPane().setVisible(true);
		if (mainImageFileChooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) {
			for (PokerStarsImageChooserListener listener : listeners) {
				listener.mainImageSelected(mainImageFileChooser
						.getSelectedFile());
			}
		}
		parent.getRootPane().getGlassPane().setVisible(false);
	}

	public void showOpenMaskImageDialog(JFrame parent) {
		parent.getRootPane().getGlassPane().setVisible(true);
		if (maskImageFileChooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) {
			for (PokerStarsImageChooserListener listener : listeners) {
				listener.maskImageSelected(maskImageFileChooser
						.getSelectedFile());
			}
		}
		parent.getRootPane().getGlassPane().setVisible(false);
	}
}
