package by.zerg.psim.helpers;

import java.io.File;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;

import org.apache.log4j.Logger;

import by.zerg.psim.exception.BaseImageManagerException;
import by.zerg.psim.exception.ImageNotFoundException;
import by.zerg.psim.exception.ImageReadException;

public class ImageHelper {
	private static final Logger LOG = Logger.getLogger(ImageHelper.class);

	public static final int WRONG_FORMAT = 0;
	public static final int BMP_FORMAT = 0x1000;
	public static final int PNG_FORMAT = 0x1100;
	public static final int BITS_8 = 8;
	public static final int BITS_24 = 24;
	public static final int BITS_32 = 32;
	public static final int BMP_8 = BMP_FORMAT | BITS_8;
	public static final int BMP_24 = BMP_FORMAT | BITS_24;
	public static final int PNG_8 = PNG_FORMAT | BITS_8;
	public static final int PNG_24 = PNG_FORMAT | BITS_24;
	public static final int PNG_32 = PNG_FORMAT | BITS_32;

	final String imageFormatName;
	final int imageFormat;
	final int imageBits;
	final int imageWidth;
	final int imageHeight;

	public ImageHelper(File imageFile) throws BaseImageManagerException {
		if (imageFile == null || !imageFile.exists()) {
			throw new ImageNotFoundException("image not found...");
		}
		ImageInputStream in = null;
		try {
			in = ImageIO.createImageInputStream(imageFile);
			ImageReader reader = ImageIO.getImageReaders(in).next();
			reader.setInput(in, true, true);
			imageWidth = reader.getWidth(0);
			imageHeight = reader.getHeight(0);
			imageFormatName = reader.getFormatName();
			imageFormat = imageFormatName.equals("bmp") ? BMP_FORMAT
					: imageFormatName.equals("png") ? PNG_FORMAT : WRONG_FORMAT;
			imageBits = reader.getImageTypes(0).next().getColorModel()
					.getPixelSize();
			reader.dispose();
		} catch (Throwable t) {
			// TODO handle this correctly
			throw new ImageReadException("can't read image type");
		} finally {
			try {
				in.close();
			} catch (Throwable t) {
				// TODO handle this correctly
				t.printStackTrace();
			}
		}
	}

	public String getImageFormatName() {
		return imageFormatName;
	}

	public int getImageFormat() {
		return imageFormat;
	}

	public int getImageBits() {
		return imageBits;
	}

	public int getImageWidth() {
		return imageWidth;
	}

	public int getImageHeight() {
		return imageHeight;
	}

	public int getImageType() {
		return imageFormat | imageBits;
	}
}
