package by.zerg.psim.model;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import org.apache.log4j.Logger;

public class Config implements Serializable {
	private static final Logger LOG = Logger.getLogger(Config.class);

	public static final String EDITOR_COMMAND_PROPERTY_DEFAULTS = "bin\\edit_image.bat";
	public static final String NAVIGATOR_COMMAND_PROPERTY_DEFAULTS = "bin\\find_image.bat";
	public static final String MSKMGR_DECODE_COMMAND_PROPERTY_DEFAULTS = "bin\\mskmgr_decode.bat";
	public static final String MSKMGR_ENCODE_COMMAND_PROPERTY_DEFAULTS = "bin\\mskmgr_encode.bat";
	public static final String PNG2BMP_CONVERT_COMMAND_PROPERTY_DEFAULTS = "bin\\png2bmp_convert.bat";
	public static final boolean AUTO_OPEN_ASSOCIATED_IMAGES_PROPERTY_DEFAULTS = true;
	public static final boolean AUTO_OPEN_EXPORTS_PROPERTY_DEFAULTS = true;
	public static final boolean AUTO_BACKUP_TARGETS_PROPERTY_DEFAULTS = true;
	public static final boolean SAVE_CURRENT_CONFIG_ON_EXIT_PROPERTY_DEFAULTS = true;
	public static final boolean REMEMBER_LAST_SELECTIONS_PROPERTY_DEFAULTS = true;
	public static final String LAST_SELECTION_FILE_PATH_PROPERTY_DEFAULTS = null;
	public static final int ORIGINAL_IMAGE_SIZING = 0;
	public static final int SCALED_IMAGE_SIZING = 1;

	private static Config instance;

	private int imageSizing;
	private String lastSelectionFilePath;
	private String editorCommand;
	private String navigatorCommand;
	private String mskmgrDecodeCommand;
	private String mskmgrEncodeCommand;
	private String png2bmpConvertCommand;
	private boolean autoOpenAssociatedImages;
	private boolean autoOpenExports;
	private boolean autoBackupTargets;
	private boolean saveCurrentConfigOnExit;
	private boolean rememberLastSelections;

	private Config() {
		if (LOG.isDebugEnabled()) {
			LOG.debug("creating new config instance...");
		}
		imageSizing = SCALED_IMAGE_SIZING;
		lastSelectionFilePath = LAST_SELECTION_FILE_PATH_PROPERTY_DEFAULTS;
		editorCommand = EDITOR_COMMAND_PROPERTY_DEFAULTS;
		navigatorCommand = NAVIGATOR_COMMAND_PROPERTY_DEFAULTS;
		mskmgrDecodeCommand = MSKMGR_DECODE_COMMAND_PROPERTY_DEFAULTS;
		mskmgrEncodeCommand = MSKMGR_ENCODE_COMMAND_PROPERTY_DEFAULTS;
		png2bmpConvertCommand = PNG2BMP_CONVERT_COMMAND_PROPERTY_DEFAULTS;
		autoOpenAssociatedImages = AUTO_OPEN_ASSOCIATED_IMAGES_PROPERTY_DEFAULTS;
		autoOpenExports = AUTO_OPEN_EXPORTS_PROPERTY_DEFAULTS;
		autoBackupTargets = AUTO_BACKUP_TARGETS_PROPERTY_DEFAULTS;
		saveCurrentConfigOnExit = SAVE_CURRENT_CONFIG_ON_EXIT_PROPERTY_DEFAULTS;
		rememberLastSelections = REMEMBER_LAST_SELECTIONS_PROPERTY_DEFAULTS;
		if (LOG.isDebugEnabled()) {
			LOG.debug("new config instance created...");
		}
	}

	public static Config getInstance() {
		if (instance == null) {
			if (LOG.isDebugEnabled()) {
				LOG.debug("loading config instance from conf\\config.instance...");
			}
			try {
				FileInputStream fileIn = new FileInputStream(
						"conf\\config.instance");
				ObjectInputStream in = new ObjectInputStream(fileIn);
				instance = (Config) in.readObject();
				in.close();
				fileIn.close();
				if (LOG.isDebugEnabled()) {
					LOG.debug("Config[conf\\config.instance] instance loaded...");
				}
			} catch (Throwable t) {
				LOG.error(
						"Config[conf\\config.instance] instance loading failed...",
						t);
				instance = new Config();
			}
		}
		return instance;
	}

	public void save() {
		if (LOG.isDebugEnabled()) {
			LOG.debug("saving config...");
		}
		try {
			FileOutputStream fileOut = new FileOutputStream(
					"conf\\config.instance");
			ObjectOutputStream out = new ObjectOutputStream(fileOut);
			out.writeObject(this);
			out.close();
			fileOut.close();
			if (LOG.isDebugEnabled()) {
				LOG.debug("config saved...");
			}
		} catch (Throwable t) {
			LOG.error("can't save Config[conf\\config.instance]...", t);
		}
	}

	public String getEditorCommand() {
		return editorCommand;
	}

	public void setEditorCommand(String editorCommand) {
		this.editorCommand = editorCommand;
	}

	public String getNavigatorCommand() {
		return navigatorCommand;
	}

	public void setNavigatorCommand(String navigatorCommand) {
		this.navigatorCommand = navigatorCommand;
	}

	public boolean isAutoOpenAssociatedImages() {
		return autoOpenAssociatedImages;
	}

	public void setAutoOpenAssociatedImages(boolean autoOpenAssociatedImages) {
		this.autoOpenAssociatedImages = autoOpenAssociatedImages;
	}

	public boolean isSaveCurrentConfigOnExit() {
		return saveCurrentConfigOnExit;
	}

	public void setSaveCurrentConfigOnExit(boolean saveCurrentConfigOnExit) {
		this.saveCurrentConfigOnExit = saveCurrentConfigOnExit;
	}

	public int getImageSizing() {
		return imageSizing;
	}

	public void setImageSizing(int imageSizing) {
		this.imageSizing = imageSizing;
	}

	public boolean isAutoOpenExports() {
		return autoOpenExports;
	}

	public void setAutoOpenExports(boolean autoOpenExports) {
		this.autoOpenExports = autoOpenExports;
	}

	public String getMskmgrDecodeCommand() {
		return mskmgrDecodeCommand;
	}

	public void setMskmgrDecodeCommand(String mskmgrDecodeCommand) {
		this.mskmgrDecodeCommand = mskmgrDecodeCommand;
	}

	public String getMskmgrEncodeCommand() {
		return mskmgrEncodeCommand;
	}

	public void setMskmgrEncodeCommand(String mskmgrEncodeCommand) {
		this.mskmgrEncodeCommand = mskmgrEncodeCommand;
	}

	public String getPng2bmpConvertCommand() {
		return png2bmpConvertCommand;
	}

	public void setPng2bmpConvertCommand(String png2bmpConvertCommand) {
		this.png2bmpConvertCommand = png2bmpConvertCommand;
	}

	public boolean isAutoBackupTargets() {
		return autoBackupTargets;
	}

	public void setAutoBackupTargets(boolean autoBackupTargets) {
		this.autoBackupTargets = autoBackupTargets;
	}

	public boolean isRememberLastSelections() {
		return rememberLastSelections;
	}

	public void setRememberLastSelections(boolean rememberLastSelections) {
		this.rememberLastSelections = rememberLastSelections;
	}

	public String getLastSelectionFilePath() {
		return lastSelectionFilePath;
	}

	public void setLastSelectionFilePath(String lastSelectionFilePath) {
		this.lastSelectionFilePath = lastSelectionFilePath;
	}
}
