package by.zerg.psim.model;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;

import by.zerg.psim.exception.BaseImageManagerException;
import by.zerg.psim.exception.ImageFileNotSelectedException;
import by.zerg.psim.exception.ImageSizeException;
import by.zerg.psim.exception.ImageTypeException;
import by.zerg.psim.helpers.BinExecutor;
import by.zerg.psim.helpers.ImageHelper;

public class ImageManager {
	private static final Logger LOG = Logger.getLogger(ImageManager.class);
	private static final File CURRENT_MAIN_IMAGE_FILE = new File(
			"images\\current_main_image.bmp");
	private static final File CURRENT_MASK_IMAGE_FILE = new File(
			"images\\current_mask_image.bmp");
	private static final File CURRENT_EXTRACTED_MASK_IMAGE_FILE = new File(
			"images\\current_extracted_mask_image.bmp");

	File selectedMainImageFile;
	File selectedMaskImageFile;
	File editableMainImageFile;
	File editableMaskImageFile;

	BufferedImage currentMainImage;
	BufferedImage currentMaskImage;
	BufferedImage currentResultImage;

	public ImageManager() {
	}
	
	private void readImage(File imageFile) throws BaseImageManagerException {
		
	}

	public void setMainImage(File imageFile) throws BaseImageManagerException {
		if (LOG.isDebugEnabled()) {
			LOG.debug("start reading main image from file: " + imageFile);
		}
		ImageHelper imageHelper = new ImageHelper(imageFile);
		switch (imageHelper.getImageType()) {
		case ImageHelper.BMP_24:
		case ImageHelper.PNG_24:
		case ImageHelper.PNG_32:
			break;
		default:
			throw new ImageTypeException(
					"selected image is not supported. select 24 bits bmp or 24/32 bits png instead...");
		}
		// if (currentMaskImage != null
		// && currentMaskImage.getWidth() != imageHelper.getImageWidth()
		// && currentMaskImage.getHeight() != imageHelper.getImageHeight()) {
		// throw new ImageSizeException(
		// "selected image has different size from current mask image");
		// }
		if (LOG.isDebugEnabled()) {
			LOG.debug("selected main image is correct");
		}
		selectedMainImageFile = imageFile;
		currentMainImage = null;
		currentResultImage = null;
	}

	public void refreshMainImage() throws BaseImageManagerException {
		setMainImage(selectedMainImageFile);
	}

	public void clearMainImage() {
		selectedMainImageFile = null;
		currentMainImage = null;
		currentResultImage = null;
	}

	public void setMaskImage(File imageFile) throws BaseImageManagerException {
		if (LOG.isDebugEnabled()) {
			LOG.debug("start reading mask image from file: " + imageFile);
		}
		ImageHelper imageHelper = new ImageHelper(imageFile);
		switch (imageHelper.getImageType()) {
		case ImageHelper.BMP_8:
		case ImageHelper.BMP_24:
		case ImageHelper.PNG_24:
		case ImageHelper.PNG_32:
			break;
		default:
			throw new ImageTypeException(
					"selected image is not supported. select 8 bits bmp instead...");
		}
		// if (currentMainImage != null
		// && currentMainImage.getWidth() != imageHelper.getImageWidth()
		// && currentMainImage.getHeight() != imageHelper.getImageHeight()) {
		// throw new ImageSizeException(
		// "selected image has different size from current main image");
		// }
		if (LOG.isDebugEnabled()) {
			LOG.debug("selected mask image is correct");
		}
		selectedMaskImageFile = imageFile;
		currentMaskImage = null;
		currentResultImage = null;
	}

	public void refreshMaskImage() throws BaseImageManagerException {
		setMaskImage(selectedMaskImageFile);
	}

	public void clearMaskImage() {
		selectedMaskImageFile = null;
		currentMaskImage = null;
		currentResultImage = null;
	}

	// public void openMainImage(File imageFile) {
	// if (LOG.isDebugEnabled()) {
	// LOG.debug("start loading main image from file: "
	// + imageFile.getAbsolutePath());
	// }
	// try {
	// ImageHelper imageHelper = new ImageHelper(imageFile);
	// if (currentMaskImage.getWidth() != imageHelper.getImageWidth()
	// || currentMaskImage.getHeight() != imageHelper
	// .getImageHeight()) {
	//
	// }
	// selectedMainImageFile = imageFile;
	// switch (imageHelper.getImageType()) {
	// case ImageHelper.PNG_32:
	// extractPng(imageFile);
	// break;
	// case ImageHelper.PNG_24:
	// nativeExtractPng();
	// break;
	// }
	//
	// } catch (Throwable t) {
	// LOG.error("can't read mainImage[ " + imageFile + " ]");
	// }
	// if (LOG.isDebugEnabled()) {
	// LOG.debug("main image loaded from file: "
	// + imageFile.getAbsolutePath());
	// }
	// }
	//
	// public void openMainImage(File mainImageFile, boolean needOpenMaskImage)
	// {
	// if (LOG.isDebugEnabled()) {
	// LOG.debug("start loading main image from file: "
	// + mainImageFile.getAbsolutePath());
	// }
	// try {
	// selectedMainImageFile = mainImageFile;
	// switch (ImageHelper.getImageType(mainImageFile)) {
	// case ImageHelper.PNG_32:
	// case ImageHelper.PNG_24:
	// case ImageHelper.PNG_8:
	// extractPng();
	// break;
	// case ImageHelper.BMP_24:
	// case ImageHelper.BMP_8:
	// FileUtils.copyFile(mainImageFile, CURRENT_MAIN_IMAGE_FILE);
	// if (needOpenMaskImage) {
	// FileUtils.copyFile(findMaskImageFile(mainImageFile),
	// CURRENT_MASK_IMAGE_FILE);
	// }
	// break;
	// }
	// currentMainImage = ImageIO.read(CURRENT_MAIN_IMAGE_FILE);
	// if (needOpenMaskImage) {
	// extractMask();
	// currentMaskImage = ImageIO
	// .read(CURRENT_EXTRACTED_MASK_IMAGE_FILE);
	// }
	// setResultImage();
	// if (LOG.isDebugEnabled()) {
	// LOG.debug("main image was successfully loaded from file: "
	// + mainImageFile.getAbsolutePath());
	// }
	// } catch (Throwable t) {
	// LOG.error("error of loading main image from file: "
	// + mainImageFile.getAbsolutePath());
	// }
	// }
	//
	// public void maskImageSelected(File maskImageFile, boolean
	// needOpenMainImage) {
	// if (LOG.isDebugEnabled()) {
	// LOG.debug("start loading mask image from file: "
	// + maskImageFile.getAbsolutePath());
	// }
	// try {
	// selectedMaskImageFile = maskImageFile;
	// if (ImageHelper.getImageType(maskImageFile) == ImageHelper.BMP_8) {
	// FileUtils.copyFile(maskImageFile, CURRENT_MASK_IMAGE_FILE);
	// extractMask();
	// }
	// currentMaskImage = ImageIO.read(CURRENT_EXTRACTED_MASK_IMAGE_FILE);
	// if (needOpenMainImage) {
	// FileUtils.copyFile(findMainImageFile(maskImageFile),
	// CURRENT_MASK_IMAGE_FILE);
	// currentMainImage = ImageIO.read(CURRENT_MAIN_IMAGE_FILE);
	// }
	// setResultImage();
	// if (LOG.isDebugEnabled()) {
	// LOG.debug("mask image was successfully loaded from file: "
	// + maskImageFile.getAbsolutePath());
	// }
	// } catch (Throwable t) {
	// LOG.error("error of loading mask image from file: "
	// + maskImageFile.getAbsolutePath());
	// }
	// }

	public File findMaskImageFile(File mainImageFile)
			throws BaseImageManagerException {
		ImageHelper imageHelper = new ImageHelper(mainImageFile);
		switch (imageHelper.getImageFormat()) {
		case ImageHelper.BMP_FORMAT:
			return new File(mainImageFile.getAbsolutePath().toLowerCase()
					.replace(".bmp", ".a.bmp"));
		case ImageHelper.PNG_FORMAT:
			return mainImageFile;
		}
		return null;
	}

	public File findMaskImageFile() throws BaseImageManagerException {
		return findMaskImageFile(selectedMainImageFile);
	}

	public File findMainImageFile(File maskImageFile)
			throws BaseImageManagerException {
		ImageHelper imageHelper = new ImageHelper(maskImageFile);
		switch (imageHelper.getImageFormat()) {
		case ImageHelper.BMP_FORMAT:
			File mainImageFile = new File(maskImageFile.getAbsolutePath()
					.toLowerCase().replace(".a.bmp", ".bmp"));
			return mainImageFile.equals(maskImageFile) ? null : mainImageFile;
		case ImageHelper.PNG_FORMAT:
			return maskImageFile;
		}
		return null;
	}

	public File findMainImageFile() throws BaseImageManagerException {
		return findMainImageFile(selectedMaskImageFile);
	}

	// public void nativeExtractPng() throws IOException {
	// BufferedImage selectedImage = ImageIO.read(selectedMainImageFile);
	// ImageIO.write(selectedImage, "bmp", CURRENT_MAIN_IMAGE_FILE);
	// Graphics2D maskImageGraphics = selectedImage.createGraphics();
	// maskImageGraphics.setBackground(Color.WHITE);
	// maskImageGraphics.clearRect(0, 0, selectedImage.getWidth(),
	// selectedImage.getHeight());
	// ImageIO.write(selectedImage, "bmp", CURRENT_EXTRACTED_MASK_IMAGE_FILE);
	// applyMask();
	// extractMask();
	// }
	//
	// public void extractPng() throws IOException {
	// int result = BinExecutor.execute(Config.getInstance()
	// .getPng2bmpConvertCommand()
	// + " \""
	// + selectedMainImageFile.getAbsolutePath()
	// + "\" \""
	// + CURRENT_MAIN_IMAGE_FILE.getAbsolutePath()
	// + "\" \""
	// + CURRENT_MASK_IMAGE_FILE.getAbsolutePath() + "\"");
	// if (result != BinExecutor.SUCCESS) {
	// nativeExtractPng();
	// }
	// }

	public void extractMask() {
		BinExecutor.execute(Config.getInstance().getMskmgrDecodeCommand()
				+ " \"" + CURRENT_MASK_IMAGE_FILE.getAbsolutePath() + "\" \""
				+ CURRENT_EXTRACTED_MASK_IMAGE_FILE.getAbsolutePath() + "\"");
	}

	public void applyMask() {
		BinExecutor.execute(Config.getInstance().getMskmgrEncodeCommand()
				+ " \"" + CURRENT_EXTRACTED_MASK_IMAGE_FILE.getAbsolutePath()
				+ "\" \"" + CURRENT_MASK_IMAGE_FILE.getAbsolutePath() + "\"");
	}

	public void convert2png(File pngFile) {
		try {
			if (LOG.isDebugEnabled()) {
				LOG.debug("start converting BMP[ "
						+ selectedMainImageFile.getAbsolutePath()
						+ " ] to PNG[ " + pngFile.getAbsolutePath() + " ]");
			}
			if (LOG.isDebugEnabled()) {
				LOG.debug("applying mask...");
			}
			BufferedImage resultImage = new BufferedImage(
					currentMainImage.getWidth(), currentMainImage.getHeight(),
					6);
			for (int x = 0; x < resultImage.getWidth(); x++) {
				for (int y = 0; y < resultImage.getHeight(); y++) {
					float maskColor = (float) -(currentMaskImage.getRGB(x, y) + 1) / 0x00FFFFFF;
					Color srcColor = new Color(currentMainImage.getRGB(x, y));
					int red = srcColor.getRed();
					int green = srcColor.getGreen();
					int blue = srcColor.getBlue();
					int alpha = (int) (255 * (1 - maskColor));
					resultImage.setRGB(x, y,
							new Color(red, green, blue, alpha).getRGB());
				}
			}
			if (LOG.isDebugEnabled()) {
				LOG.debug("saving file...");
			}
			ImageIO.write(resultImage, "png", pngFile);
		} catch (Throwable t) {
			LOG.error("unhandled exception while converting to PNG", t);
		}
	}

	public void convert2bmp(File bmpFile) throws BaseImageManagerException {
		BinExecutor.execute(Config.getInstance().getPng2bmpConvertCommand()
				+ " \"" + selectedMainImageFile.getAbsolutePath() + " \" \""
				+ bmpFile.getAbsolutePath() + " \" \""
				+ findMaskImageFile(bmpFile) + "\"");
	}

	// private void setResultImage() {
	// try {
	// currentResultImage = new BufferedImage(currentMainImage.getWidth(),
	// currentMainImage.getHeight(), 6);
	// for (int x = 0; x < currentMainImage.getWidth(); x++) {
	// for (int y = 0; y < currentMainImage.getHeight(); y++) {
	// float maskColor = (float) -(currentMaskImage.getRGB(x, y) + 1) /
	// 0x00FFFFFF;
	// Color srcColor = new Color(currentMainImage.getRGB(x, y));
	// int red = srcColor.getRed();
	// int green = srcColor.getGreen();
	// int blue = srcColor.getBlue();
	// int alpha = (int) (255 * (1 - maskColor));
	// currentResultImage.setRGB(x, y, new Color(red, green, blue,
	// alpha).getRGB());
	// }
	// }
	// } catch (Throwable t) {
	// LOG.error("unhandled exception while applying mask...", t);
	// currentResultImage = null;
	// }
	// }

	public BufferedImage getCurrentMainImage() throws BaseImageManagerException {
		if (currentMainImage != null) {
			return currentMainImage;
		}
		if (selectedMainImageFile == null) {
			throw new ImageFileNotSelectedException(
					"Main image file not selected...");
		}
		ImageHelper imageHelper = new ImageHelper(selectedMainImageFile);
		switch (imageHelper.getImageType()) {
		case ImageHelper.BMP_8:
		case ImageHelper.BMP_24:
			try {
				FileUtils.copyFile(selectedMainImageFile,
						CURRENT_MAIN_IMAGE_FILE);
				currentMainImage = ImageIO.read(CURRENT_MAIN_IMAGE_FILE);
			} catch (Throwable t) {
				t.printStackTrace();
				// TODO handle this
			}
			break;
		case ImageHelper.PNG_24:
			try {
				currentMainImage = ImageIO.read(selectedMainImageFile);
				ImageIO.write(currentMainImage, "bmp", CURRENT_MAIN_IMAGE_FILE);
			} catch (Throwable t) {
				t.printStackTrace();
				// TODO handle this
			}
			break;
		case ImageHelper.PNG_32:
			try {
				BinExecutor.execute(Config.getInstance()
						.getPng2bmpConvertCommand()
						+ " \""
						+ selectedMainImageFile.getAbsolutePath()
						+ "\" \""
						+ CURRENT_MAIN_IMAGE_FILE.getAbsolutePath() + "\"");
				currentMainImage = ImageIO.read(CURRENT_MAIN_IMAGE_FILE);
			} catch (Throwable t) {
				t.printStackTrace();
				// TODO handle this
			}
			break;
		}
		return currentMainImage;
	}

	public BufferedImage getCurrentMaskImage() throws BaseImageManagerException {
		if (currentMaskImage != null) {
			return currentMaskImage;
		}
		if (selectedMaskImageFile == null) {
			throw new ImageFileNotSelectedException(
					"Mask image file not selected...");
		}
		ImageHelper imageHelper = new ImageHelper(selectedMaskImageFile);
		switch (imageHelper.getImageType()) {
		case ImageHelper.BMP_8:
			try {
				FileUtils.copyFile(selectedMaskImageFile,
						CURRENT_MASK_IMAGE_FILE);
				extractMask();
				currentMaskImage = ImageIO
						.read(CURRENT_EXTRACTED_MASK_IMAGE_FILE);
			} catch (Throwable t) {
				t.printStackTrace();
				// TODO handle this
			}
			break;
		case ImageHelper.PNG_24:
			try {
				currentMaskImage = ImageIO.read(selectedMaskImageFile);
				Graphics2D maskImageGraphics = currentMaskImage
						.createGraphics();
				maskImageGraphics.setBackground(Color.WHITE);
				maskImageGraphics.clearRect(0, 0, currentMaskImage.getWidth(),
						currentMaskImage.getHeight());
				ImageIO.write(currentMaskImage, "bmp",
						CURRENT_EXTRACTED_MASK_IMAGE_FILE);
				applyMask();
				extractMask();
			} catch (Throwable t) {
				t.printStackTrace();
				// TODO handle this
			}
			break;
		case ImageHelper.PNG_32:
			try {
				BinExecutor.execute(Config.getInstance()
						.getPng2bmpConvertCommand()
						+ " \""
						+ selectedMaskImageFile.getAbsolutePath()
						+ "\" \""
						+ CURRENT_MAIN_IMAGE_FILE.getAbsolutePath()
						+ "\" \""
						+ CURRENT_MASK_IMAGE_FILE + "\"");
				extractMask();
				currentMaskImage = ImageIO
						.read(CURRENT_EXTRACTED_MASK_IMAGE_FILE);
			} catch (Throwable t) {
				t.printStackTrace();
				// TODO handle this
			}
		}
		return currentMaskImage;
	}

	public BufferedImage getCurrentResultImage()
			throws BaseImageManagerException {
		if (currentResultImage != null) {
			return currentResultImage;
		} else if (currentMainImage == null) {
			return null;
		} else if (currentMaskImage == null) {
			currentResultImage = currentMainImage;
			return currentResultImage;
		}
		if (currentMainImage.getWidth() != currentMaskImage.getWidth()
				|| currentMainImage.getHeight() != currentMaskImage.getHeight()) {
			throw new ImageSizeException(
					"main and mask images have different sizes");
		}
		try {
			currentResultImage = new BufferedImage(currentMainImage.getWidth(),
					currentMainImage.getHeight(), 6);
			for (int x = 0; x < currentMainImage.getWidth(); x++) {
				for (int y = 0; y < currentMainImage.getHeight(); y++) {
					float maskColor = (float) -(currentMaskImage.getRGB(x, y) + 1) / 0x00FFFFFF;
					Color srcColor = new Color(currentMainImage.getRGB(x, y));
					int red = srcColor.getRed();
					int green = srcColor.getGreen();
					int blue = srcColor.getBlue();
					int alpha = (int) (255 * (1 - maskColor));
					currentResultImage.setRGB(x, y, new Color(red, green, blue,
							alpha).getRGB());
				}
			}
		} catch (Throwable t) {
			// TODO handle this
			LOG.error("unhandled exception while applying mask...", t);
			currentResultImage = null;
		}
		return currentResultImage;
	}

	public File getSelectedMainImageFile() {
		return selectedMainImageFile;
	}

	public File getSelectedMaskImageFile() {
		return selectedMaskImageFile;
	}

	public File getEditableMainImageFile() {
		return CURRENT_MAIN_IMAGE_FILE;
	}

	public File getEditableMaskImageFile() {
		return CURRENT_EXTRACTED_MASK_IMAGE_FILE;
	}
}
